'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _storage = require('./storage');

var _generateToken = require('./generate-token');

var _generateToken2 = _interopRequireDefault(_generateToken);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var STATE_KEY = 'state';
var NONCE_KEY = 'nonce';
var CREDENTIALS_KEY = 'credentials';

// Lightweight check: has the user just been redirected from authenticating at Flux?
// The implicit OIDC flow will stick access_token (among other things) in URL hash.
function isLoggingIn() {
  return window.location.hash.search(/access_token/) !== -1;
}

var FluxHelpers = function () {
  function FluxHelpers(sdk) {
    _classCallCheck(this, FluxHelpers);

    if (!sdk) {
      throw new Error('Must supply an SDK');
    }

    this._sdk = sdk;
  }

  _createClass(FluxHelpers, [{
    key: 'redirectToFluxLogin',
    value: function redirectToFluxLogin(replace) {
      // We clear up local storage in case there's a lingering user in there.
      (0, _storage.clear)();

      var state = (0, _generateToken2.default)();
      var nonce = (0, _generateToken2.default)();

      (0, _storage.store)(STATE_KEY, state);
      (0, _storage.store)(NONCE_KEY, nonce);

      var url = this._sdk.getAuthorizeUrl(state, nonce);

      // We need to assign and call the redirect function like this, or the browser
      // may throw an error due to special window.location behaviour.
      var redirect = replace ? 'replace' : 'assign';
      window.location[redirect](url);
    }
  }, {
    key: 'storeFluxUser',
    value: function storeFluxUser(redirectUrl, replace) {
      var _this = this;

      var promise = null;

      if (!isLoggingIn()) {
        promise = Promise.resolve();
      } else {
        (function () {
          var state = (0, _storage.retrieve)(STATE_KEY);
          var nonce = (0, _storage.retrieve)(NONCE_KEY);

          // As above, we need to assign and call the redirect function specially or the browser
          // may throw an error due to special window.location behaviour.
          var redirect = replace ? 'replace' : 'assign';
          var url = redirectUrl || window.location.href.replace(window.location.hash, '');

          if (!state) {
            throw new Error('No `state` stored');
          } else if (!nonce) {
            throw new Error('No `nonce` stored');
          }

          promise = _this._sdk.exchangeCredentials(state, nonce).then(function (credentials) {
            // We clear here to get rid of the now-unneeded `state` and `nonce` values,
            // as well as anything else that might be lingering in storage.
            (0, _storage.clear)();

            (0, _storage.store)(CREDENTIALS_KEY, credentials);
          }).then(function () {
            window.location[redirect](url);
          });
        })();
      }

      return promise;
    }
  }, {
    key: 'isLoggedIn',
    value: function isLoggedIn() {
      var credentials = (0, _storage.retrieve)(CREDENTIALS_KEY);

      return this._sdk.User.isLoggedIn(credentials).then(function (loggedIn) {
        if (!loggedIn) {
          // We clear the credentials key in case invalid (e.g., expired) credentials are
          // still in there.
          (0, _storage.clear)(CREDENTIALS_KEY);
        }

        return loggedIn;
      });
    }
  }, {
    key: 'logout',
    value: function logout() {
      // So long and thanks for all the fish!
      (0, _storage.clear)();
    }
  }, {
    key: 'getUser',
    value: function getUser() {
      var credentials = (0, _storage.retrieve)(CREDENTIALS_KEY);

      if (!credentials) {
        throw new Error('Cannot get current user: No user credentials stored');
      }

      return this._sdk.getUser(credentials);
    }

    /* eslint-disable consistent-return */

  }, {
    key: 'login',
    value: function login(redirectUrl, replace) {
      if (!isLoggingIn()) {
        // We don't return anything here because the user is synchronously redirected,
        // interrupting the current browser execution.
        this.redirectToFluxLogin(replace);
      } else {
        return this.storeFluxUser(redirectUrl, replace);
      }
    }
    /* eslint-enable consistent-return */

  }]);

  return FluxHelpers;
}();

exports.default = FluxHelpers;
//# sourceMappingURL=helpers.js.map